@@ -462,34 +462,35 @@ pdfclustCBI <- function(data,...){
 }
 
 # distr could also be mvt, msn, mst
-emskewCBI <- function(data,k,distr="mst",repeats=100,...){
-  if (requireNamespace("EMMIXskew", quietly = TRUE)) {
-    if (!exists("distr")) distr="mvn" 
-    attempt <- 1
-    repeat{
-      c1 <- EMMIXskew::EmSkew(data,g=k,distr=distr,...)
-      if (!is.null(c1)){
-        if (max(c1$clust)==k)
-          break
-      }
-      attempt <- attempt+1
-      if(attempt>repeats){
-        cat("EmSkew failed after ",repeats," repetitions.")
-        break
-      }
-      if(debug) print("Repeat EmSkew exection")    
-    }
-    partition <- c1$clust
-    nc <- max(partition)
-    cl <- list()
-    for (i in 1:nc) cl[[i]] <- partition == i
-    out <- list(result = c1, nc = nc, clusterlist = cl, partition = partition, 
-          clustermethod = paste("emskew_",distr,sep=""))
-    out
-  }
-  else
-    warning("EMMIXskew could not be loaded")    
-}
+# emskewCBI <- function(data,k,distr="mst",repeats=100,...){
+#   if (requireNamespace("EMMIXskew", quietly = TRUE)) {
+#     if (!exists("distr")) distr="mvn" 
+#     attempt <- 1
+#     repeat{
+# #      c1 <- EMMIXskew::EmSkew(data,g=k,distr=distr,...)
+#       c1 <- EmSkew(data,g=k,distr=distr,...)
+#       if (!is.null(c1)){
+#         if (max(c1$clust)==k)
+#           break
+#       }
+#       attempt <- attempt+1
+#       if(attempt>repeats){
+#         cat("EmSkew failed after ",repeats," repetitions.")
+#         break
+#       }
+#       if(debug) print("Repeat EmSkew exection")    
+#     }
+#     partition <- c1$clust
+#     nc <- max(partition)
+#     cl <- list()
+#     for (i in 1:nc) cl[[i]] <- partition == i
+#     out <- list(result = c1, nc = nc, clusterlist = cl, partition = partition, 
+#           clustermethod = paste("emskew_",distr,sep=""))
+#     out
+#   }
+#   else
+#     warning("EMMIXskew could not be loaded")    
+# }
 
 
 
@@ -995,104 +996,168 @@ disthclusttreeCBI <- function(dmatrix,minlevel=2,method,...){
   out
 }
 
-
-classifnp <- function (data, clustering, method = "centroid", cdist = NULL, 
-    centroids = NULL, nnk = 1) 
-{
-    data <- as.matrix(data)
-    k <- max(clustering)
-    p <- ncol(data)
-    n <- nrow(data)
-    topredict <- clustering < 0
-    if (method == "averagedist") {
-        if (is.null(cdist)) 
-            cdist <- as.matrix(dist(data))
-        else cdist <- as.matrix(cdist)
-        prmatrix <- matrix(0, ncol = k, nrow = sum(topredict))
-        for (j in 1:k) prmatrix[, j] <- rowMeans(as.matrix(cdist[topredict, 
-            clustering == j]))
-        clpred <- apply(prmatrix, 1, which.min)
-        clustering[topredict] <- clpred
-    }
-    if (method == "centroid") {
-        if (is.null(centroids)) {
-            centroids <- matrix(NA, ncol = p, nrow = k)
-            grvec <- numeric(0)
-            for (j in 1:k)
-              if (sum(clustering==j)>0){
-                   centroids[j, ] <- colMeans(as.matrix(data[clustering == j, ]))
-                   grvec <- c(grvec,j)
-              }
-        }
-        else
-          grvec <- 1:k
-#        print(centroids)
-        clustering[topredict] <- knn1(centroids[grvec,], data[topredict, 
-            ], grvec)
-    }
-    if (method == "qda") {
-        qq <- try(qda(data[!topredict, ], grouping = as.factor(clustering[!topredict])), 
-            silent = TRUE)
-        if (identical(attr(qq, "class"), "try-error")) 
-            qq <- lda(data[!topredict, ], grouping = as.factor(clustering[!topredict]))
-        clustering[topredict] <- as.integer(predict(qq, data[topredict, 
-            ])$class)
-    }
-    if (method == "knn") 
-        clustering[topredict] <- as.integer(knn(data[!topredict, 
-            ], data[topredict, ], as.factor(clustering[!topredict]), 
-            k = nnk))
-    clustering
-}
-
-
-
-# method centroids doesn't allow centroids=NULL
-classifdist <- function(cdist,clustering,
-                      method="averagedist",
-                      centroids=NULL,nnk=1){
+# New with "fn" method 
+classifdist <- function (cdist, clustering, method = "averagedist", centroids = NULL, nnk = 1) {
   k <- max(clustering)
-  n <- nrow(data)
+  n <- nrow(cdist)
   cdist <- as.matrix(cdist)
-  topredict <- clustering<0
-  if(method=="averagedist"){
-    prmatrix <- matrix(0,ncol=k,nrow=sum(topredict))
-    for (j in 1:k)
-      prmatrix[,j] <- rowMeans(as.matrix(cdist[topredict,clustering==j]))
-    clpred <- apply(prmatrix,1,which.min)
+  topredict <- clustering < 0
+  if (method == "averagedist") {
+    prmatrix <- matrix(0, ncol = k, nrow = sum(topredict))
+#    print("classifdist")
+#    print(topredict)
+#    print(clustering)
+#    print(prmatrix)
+    for (j in 1:k) {
+#      print(j)
+#      print(cdist[topredict, clustering == j,drop=FALSE])
+      if (sum(clustering==j)>0)
+        prmatrix[, j] <- rowMeans(as.matrix(cdist[topredict, clustering == j,drop=FALSE]))
+      else
+        prmatrix[, j] <- rep(Inf,sum(topredict))
+    }
+    clpred <- apply(prmatrix, 1, which.min)
     clustering[topredict] <- clpred
   }
-# print("centroids&cdist")
-# print(centroids)
-#  print(cdist[topredict,centroids,drop=FALSE])
-  if(method=="centroid")
-    clustering[topredict] <- apply(cdist[topredict,centroids,drop=FALSE],1,which.min)
-  if(method=="knn"){
-    cdist[topredict,topredict] <- max(cdist)+1
-    if (nnk==1){
-      bestobs <- apply(cdist[topredict,,drop=FALSE],1,which.min)
+#  print(method)
+#  print(str(centroids))
+#  print(str(topredict))
+#  print(str(cdist))
+#  print(str(clustering))
+#  if (method == "MSS") {
+#    RowMSS <- function(x) {
+#      rowSums((x - rowMeans(x))^2)
+#    }
+#    prmatrix <- matrix(0, ncol = k, nrow = sum(topredict))
+#    for (j in 1:k) {
+#      prmatrix[, j] <- RowMSS(as.matrix(cdist[topredict, clustering == j]))
+#    }
+#    clpred <- apply(prmatrix, 1, which.min)
+#    clustering[topredict] <- clpred
+#  }
+  if (method == "centroid"){
+    clustering[topredict] <- apply(cdist[topredict, centroids, 
+                                         drop = FALSE], 1, which.min)
+  }
+  if (method == "knn") {
+    cdist[topredict, topredict] <- max(cdist) + 1
+    if (nnk == 1) {
+      bestobs <- apply(cdist[topredict, , drop = FALSE], 
+                       1, which.min)
       clustering[topredict] <- clustering[bestobs]
     }
-    else{
-      for(i in (1:n)[topredict]){
-        bestobs <- order(cdist[i,])[1:nnk]
+    else {
+      for (i in (1:n)[topredict]) {
+        bestobs <- order(cdist[i, ])[1:k]
         clasum <- numeric(0)
-        for (j in 1:k)
-          clasum[j] <- sum(clustering[bestobs]==j)
+        for (j in 1:k) clasum[j] <- sum(clustering[bestobs] == 
+                                          j)
         clustering[i] <- which.max(clasum)
       }
     }
   }
+  if (method == "fn") {
+    fdist <- matrix(0, nrow=sum(topredict), ncol = k)
+    for (i in 1:k) {
+      if (sum(clustering==i)>0)
+        fdist[,i] <- apply(as.matrix(cdist[topredict, clustering==i, drop=FALSE]), 1, max)
+      else
+        fdist[,i] <- rep(Inf,sum(topredict))      
+    }
+    bestobs1 <- apply(fdist, 1, which.min)
+    clustering[topredict] <- bestobs1
+  }
   clustering
 }
+
+# New with "fn" and "lda" method
+classifnp <- function (data, clustering, method = "centroid", 
+                        cdist = NULL, centroids = NULL, nnk = 1){
+#  require(class)
+#  require(MASS)
+  data <- as.matrix(data)
+  k <- max(clustering)
+  p <- ncol(data)
+  n <- nrow(data)
+  topredict <- clustering < 0
+  
+  ##### Average linkage 
+  if (method == "averagedist") {
+    if (is.null(cdist)) { cdist <- as.matrix(dist(data)) } 
+    else                { cdist <- as.matrix(cdist) }
     
-nselectboot <- function (data, B = 50, distances = inherits(data, "dist"), clustermethod = NULL, 
-    classification = "averagedist", krange = 2:10, count = FALSE, 
-    nnk = 1, ...) 
+    prmatrix <- matrix(0, ncol = k, nrow = sum(topredict))
+#    print("classifnp")
+#    print(topredict)
+#    print(clustering)
+    for (j in 1:k){
+      prmatrix[, j] <- rowMeans(as.matrix(cdist[topredict, clustering == j,drop=FALSE]))
+    }
+    clpred <- apply(prmatrix, 1, which.min)
+    clustering[topredict] <- clpred
+  }
+  
+  #### Kmeans, PAM, specc, ...
+  if (method == "centroid") {
+    if (is.null(centroids)) {
+      centroids <- matrix(0, ncol = p, nrow = k)
+      for (j in 1:k) centroids[j, ] <- colMeans(as.matrix(data[clustering == j, ]))
+    }
+#    print(centroids)
+#    print(sum(topredict))
+    clustering[topredict] <- knn1(centroids, data[topredict,], 1:k)
+  }
+  
+  #### Mclust
+  if (method == "qda") {
+    qq <- try(qda(data[!topredict, ], grouping = as.factor(clustering[!topredict])), silent = TRUE)
+    
+    if (identical(attr(qq, "class"), "try-error")) 
+      qq <- lda(data[!topredict, ], grouping = as.factor(clustering[!topredict]))
+    clustering[topredict] <- as.integer(predict(qq, data[topredict, ])$class)
+   
+  }
+  if (method == "lda") {
+      qq <- lda(data[!topredict, ], grouping = as.factor(clustering[!topredict]))
+      clustering[topredict] <- as.integer(predict(qq, data[topredict, ])$class)
+  }
+  ### Single linkage, specClust
+  if (method == "knn"){
+    clustering[topredict] <- as.integer(knn(data[!topredict, ], 
+                                            data[topredict, ], 
+                                            as.factor(clustering[!topredict]), 
+                                            k = nnk))
+  } 
+  
+  #### Complete linkage
+  if (method == "fn") {
+    if (is.null(cdist)){ cdist <- as.matrix(dist(data)) } 
+    else               { cdist <- as.matrix(cdist)      }
+    
+    fdist <- matrix(0, nrow=sum(topredict), ncol = k)
+    for (i in 1:k) {
+      fdist[,i] <- apply(as.matrix(cdist[topredict, clustering==i]), 1, max)
+    }
+    bestobs1 <- apply(fdist, 1, which.min)
+    clustering[topredict] <- bestobs1
+  }
+  clustering
+}
+
+
+# Introduces parameter centroidname to indicate where in CBIoutput$result
+# the centroids can be found (automatically for kmeansCBI, claraCBI);
+# If NULL and data are not distances, mean is computed within classifnp
+# largeisgood: Take 1-original value so that large is good (only stabk)
+nselectboot <- function (data, B = 50, distances = inherits(data, "dist"),
+                         clustermethod = NULL, 
+    classification = "averagedist", centroidname = NULL,
+    krange = 2:10, count = FALSE, 
+    nnk = 1, largeisgood=FALSE,...) 
 {
     dista <- distances
     data <- as.matrix(data)
-    if (classification == "average") {
+    if (classification == "averagedist") {
         if (dista) 
             dmat <- data
         else dmat <- as.matrix(dist(data))
@@ -1115,154 +1180,78 @@ nselectboot <- function (data, B = 50, distances = inherits(data, "dist"), clust
                 dmat1 <- data[d1, ]
                 dmat2 <- data[d2, ]
             }
-#            print("start clustermethod")
-#           if ("diss" %in% names(formals(clustermethod)) & distances){
             if (distances){
-              clm1 <- clustermethod(as.dist(dmat1), k = k, diss=TRUE, ...)
-              clm2 <- clustermethod(as.dist(dmat2), k = k, diss=TRUE, ...)
+              if ("diss" %in% names(formals(clustermethod))){
+                clm1 <- clustermethod(as.dist(dmat1), k = k, 
+                  diss = TRUE, ...)
+                clm2 <- clustermethod(as.dist(dmat2), k = k, 
+                  diss = TRUE, ...)
+              }
+              else{
+                clm1 <- clustermethod(as.dist(dmat1), k = k, ...)
+                clm2 <- clustermethod(as.dist(dmat2), k = k, ...)
+              }
             }
-            else{
-              clm1 <- clustermethod(dmat1, k = k, ...)
-#            cl1 <- clm1$partition
-#            print("done 1")
-              clm2 <- clustermethod(dmat2, k = k, ...)
+            else {
+                clm1 <- clustermethod(dmat1, k = k, ...)
+                clm2 <- clustermethod(dmat2, k = k, ...)
             }
-            cl2 <- clm2$partition
-#            print("done 2")
+#            cl2 <- clm2$partition
+            centroids1 <- centroids2 <- NULL
             cj1 <- cj2 <- rep(-1, n)
-#            print(d1)            
-#            print(clm1$partition)            
-#            print(d2)            
-#            print(clm2$partition)            
             cj1[d1] <- clm1$partition
             cj2[d2] <- clm2$partition
-#            browser()
+#            centroids <- NULL
+            if (classification == "centroid") {
+              if (is.null(centroidname)){
+                if (identical(clustermethod, kmeansCBI))
+                  centroidname <- "centers"
+                if (identical(clustermethod, claraCBI))
+                   centroidname <- "medoids"
+              }
+              if (!is.null(centroidname)){
+                centroids1 <- clm1$result[centroidname][[1]]
+                centroids2 <- clm2$result[centroidname][[1]]
+              }
+            }
+ #         print(centroidname)
+ #           if (classification == "centroid" & dista){
+ #           centroids1 <- unlist(centroids1)
+ #           centroids2 <- unlist(centroids2,recursive=FALSE)
+ #           }
+#          print(str(clm1))
+#          print(str(centroids1))
             if (dista) {
+#              print("classifdist")
                 cj1 <- classifdist(data, cj1, method = classification, 
-                  centroids = clm1$result$medoids, nnk = nnk)
+                  centroids = centroids1, nnk = nnk)
+#              print(cj1)
+#              print(cj2)
+#              print(centroids1)
+#              print(centroids2)
                 cj2 <- classifdist(data, cj2, method = classification, 
-                  centroids = clm2$result$medoids, nnk = nnk)
+                  centroids = centroids2, nnk = nnk)
             }
             else {
-                centroids <- NULL
-                if (classification == "centroid") {
-                  if (identical(clustermethod, kmeansCBI)){ 
-                    centroids1 <- clm1$result$centers
-                    centroids2 <- clm2$result$centers
-                  }
-                  if (identical(clustermethod, claraCBI)){ 
-                    centroids1 <- clm1$result$medoids
-                    centroids2 <- clm2$result$medoids
-                  }
-                }
-#                print("classifnp")
                 cj1 <- classifnp(data, cj1, method = classification, 
                   centroids = centroids1, nnk = nnk)
-#                print("done 1")
                 cj2 <- classifnp(data, cj2, method = classification, 
                   centroids = centroids2, nnk = nnk)
-#                plot(data,col=cj1,pch=cj1,main=paste(k,"clusters a -",i))
-#                plot(data,col=cj2,pch=cj2,main=paste(k,"clusters b -",i))
-#                print("done 2")
-#                browser()
             }
-#            print(" for n loop")
-#            j <- 1
-            ctable <- table(cj1,cj2)
+            ctable <- table(cj1, cj2)
             nck1 <- rowSums(ctable)
-            stab[i,k] <- sum(nck1^2-rowSums(ctable^2))
-#            browser()
-#            for(j in 1:k)
-#             for(j in 1:n){
-#               cj1e <- cj1==cj1[j]
-#               cj1u <- !cj1e
-#               cj2e <- cj2==cj2[j]
-#               cj2u <- !cj2e
-#               stab[i,k] <- stab[i,k]+sum(cj1e & cj2u)+sum(cj2e & cj1u)
-#              j <- j+1 }
-#            print("done")
-        } # for i
-    } # for k
+            stab[i, k] <- sum(nck1^2 - rowSums(ctable^2))
+        }
+    }
     stab <- stab/n^2
     stabk <- rep(NA, max(krange))
-#   browser()
     for (k in krange) stabk[k] <- mean(stab[, k])
     kopt <- which.min(stabk)
+    if (largeisgood){
+      stabk <- 1-stabk
+    }
     out <- list(kopt = kopt, stabk = stabk, stab = stab)
 }
+
+
  
-        
-# nselectboot <- function (data, B = 50, distances = inherits(data, "dist"), clustermethod = NULL, 
-#     classification = "averagedist", krange = 2:10, count = FALSE, 
-#     nnk = 1, ...) 
-# {
-#     dista <- distances
-#     data <- as.matrix(data)
-#     if (classification == "average") {
-#         if (dista) 
-#             dmat <- data
-#         else dmat <- as.matrix(dist(data))
-#     }
-#     stab <- matrix(0, nrow = B, ncol = max(krange))
-#     n <- nrow(data)
-#     for (k in krange) {
-#         if (count) 
-#             cat(k, " clusters\n")
-#         for (i in 1:B) {
-#             if (count) 
-#                 print(i)
-#             d1 <- sample(n, n, replace = TRUE)
-#             d2 <- sample(n, n, replace = TRUE)
-#             if (dista) {
-#                 dmat1 <- data[d1, d1]
-#                 dmat2 <- data[d2, d2]
-#             }
-#             else {
-#                 dmat1 <- data[d1, ]
-#                 dmat2 <- data[d2, ]
-#             }
-#             clm1 <- clustermethod(dmat1, k = k, ...)
-#             clm2 <- clustermethod(dmat2, k = k, ...)
-#             cj1 <- cj2 <- rep(-1, n)
-#             cj1[d1] <- clm1$partition
-#             cj2[d2] <- clm2$partition
-#             if (dista) {
-#                 cj1 <- classifdist(data, cj1, method = classification, 
-#                   centroids = clm1$result$medoids, nnk = nnk)
-#                 cj2 <- classifdist(data, cj2, method = classification, 
-#                   centroids = clm2$result$medoids, nnk = nnk)
-#             }
-#             else {
-#                 centroids <- NULL
-#                 if (classification == "centroid") {
-#                   if (identical(clustermethod, kmeansCBI)){ 
-#                     centroids1 <- clm1$result$centers
-#                     centroids2 <- clm2$result$centers
-#                   }
-#                   if (identical(clustermethod, claraCBI)){ 
-#                     centroids1 <- clm1$result$medoids
-#                     centroids2 <- clm2$result$medoids
-#                   }
-#                 }
-#                 cj1 <- classifnp(data, cj1, method = classification, 
-#                   centroids = centroids1, nnk = nnk)
-#                 cj2 <- classifnp(data, cj2, method = classification, 
-#                   centroids = centroids2, nnk = nnk)
-#             }
-#             j <- 1
-#             if (j<=n){
-#               cj1e <- cj1==cj1[j]
-#               cj1u <- !cj1e
-#               cj2e <- cj2==cj2[j]
-#               cj2u <- !cj2e
-#               stab[i,k] <- stab[i,k]+sum(cj1e & cj2u)+sum(cj2e & cj1u)
-#               j <- j+1
-#             }
-#         }
-#     }
-#     stab <- stab/n^2
-#     stabk <- rep(NA, max(krange))
-#     for (k in krange) stabk[k] <- mean(stab[, k])
-#     kopt <- which.min(stabk)
-#     out <- list(kopt = kopt, stabk = stabk, stab = stab)
-# }
